/*
 * Copyright (c) 2009 The openGion Project.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.opengion.hayabusa.db;

/**
 * カラム名配列や、値配列が設定される都度呼び出される リスナーインターフェースです。
 * 
 * これは、org.opengion.hayabusa.io.TableReader 等で、テーブルの読み取り処理を
 * 行う都度呼び出されるリスナーとして使用できます。
 *
 * 想定として、adjustColumns(読取元ファイルのデータ変換を行う) や、
 * checkColumns(読取元ファイルの整合性チェックを行う) 、nullCheck(NULL チェックすべきカラム列)
 * や、DirectTableInsertTag 等で、直接DB登録を行ったりするのを、外部から設定できます。
 *
 * @og.rev 6.2.1.0 (2015/03/13) ColumnActionListener 新規作成
 * @og.group データ処理
 *
 * @version  6.2
 * @author   Kazuhiko Hasegawa
 * @since    JDK8.0,
 */
public interface ColumnActionListener {

	/**
	 * カラム名の配列が設定された場合に、呼び出されます。
	 *
	 * @og.rev 6.2.1.0 (2015/03/13) ColumnActionListener 新規作成
	 *
	 * @param names カラム名配列
	 * @return	引数カラムに対して、有効なカラムの数
	 */
	int setNames( final String[] names );

//	/**
//	 * カラム名の配列が設定された場合に、有効なカラムオブジェクト配列のアドレスを返します。
//	 * これは、#setNames(String[]) が呼ばれるまで、不定です。
//	 * #setNames(String[]) で指定されたカラム名配列の数と同じ数だけのint配列が作成されます。
//	 * その値は、整数の場合は、#setNames(String[]) の戻り値のカラムオブジェクト配列の位置番号になります。
//	 * 戻り値のカラムに含まれない場合(omitClms等で未使用になったカラム)は、-1 がセットされています。
//	 *
//	 * @og.rev 6.2.1.0 (2015/03/13) ColumnActionListener 新規作成
//	 *
//	 * @return	カラム名配列に対して、有効なカラムオブジェクト配列のアドレス
//	 */
//	int[] getColumnNos();

	/**
	 * １行分のデータが設定された場合に、呼び出されます。
	 *
	 * @og.rev 6.2.1.0 (2015/03/13) ColumnActionListener 新規作成
	 *
	 * @param values １行分のデータ配列
	 * @param rowNo  行番号
	 * @return	引数設定値に対して、変換等を行った、設定値配列
	 */
	String[] setValues( final String[] values, final int rowNo );
}
